(* Thresholds *)

val epsilon          : float
(** Value below which a float is considered null. Default is \epsilon = 2.10^{-15}. *)

val integrals_cutoff : float
(** Cutoff value for integrals. Default is \epsilon. *)


(** Mathematical constants  *)
val pi             : float
(** $\pi = 3.141~592~653~589~793~12$ *)

val two_pi         : float
(** $2 \pi$ *)

val sq_pi          : float
(** $\sqrt{\pi}$ *)

val sq_pi_over_two : float
(** $\sqrt{\pi} / 2$ *)

val pi_inv         : float
(** $1 / \pi$ *)

val two_over_sq_pi : float
(** $2 / \sqrt{\pi}$ *)


(** Physical constants *)

val a0       : float
(** Bohr's radius : $a_0 = 0.529~177~210~67(23)$ angstrom. *)

val a0_inv   : float
(** $1 / a_0$ *)

val ha_to_ev : float
(** Hartree to eV conversion factor : $27.211~386~02(17)$ *)

val ev_to_ha : float
(** eV to Hartree conversion factor : 1 / ~ha_to_ev~ *)

